@@ -1449,6 +1449,21 @@ class Trimesh(Geometry3D):
         populate = self.face_adjacency_radius
         return self._cache['face_adjacency_span']
 
+    @caching.cache_decorator
+    def integral_mean_curvature(self):
+        """
+        The integral mean curvature, or the surface integral of the mean curvature.
+
+        Returns
+        ---------
+        area : float
+          Integral mean curvature of mesh
+        """
+        edges_length = np.linalg.norm(np.subtract(
+            *self.vertices[self.face_adjacency_edges.T]), axis=1)
+        imc = (self.face_adjacency_angles * edges_length).sum() * 0.5
+        return imc
+
     @caching.cache_decorator
     def vertex_adjacency_graph(self):
         """
@@ -1902,7 +1917,7 @@ class Trimesh(Geometry3D):
             # get a copy of the current visuals
             visual = self.visual.copy()
 
-            # seperate uv coords and vertices
+            # separate uv coords and vertices
             vertices, visual.uv = vertices[:, :3], vertices[:, 3:]
 
         else:
@@ -1959,7 +1974,7 @@ class Trimesh(Geometry3D):
             # get a copy of the current visuals
             visual = self.visual.copy()
 
-            # seperate uv coords and vertices
+            # separate uv coords and vertices
             vertices, visual.uv = vertices[:, :3], vertices[:, 3:]
 
         else:
